/* Interface for ButtonCell class
 *
 * Copyright (C)  1993  The Board of Trustees of  
 * The Leland Stanford Junior University.  All Rights Reserved.
 *
 * Authors: Scott Francis, Fred Harris, Paul Kunz, Imran Qureshi,
 * and Libing Wang.
 *
 * This file is part of an Objective-C class library for a window system
 *
 * ButtonCell.h,v 1.37 1995/12/13 22:32:53 fedor Exp
 */

 
#ifndef _ButtonCell_h_
#define _ButtonCell_h_

#include "ActionCell.h"
#include "sysparams.h"

#define NXBUTTON 0
#define NXRADIO 1
#define NXSWITCH 2

/* Button Types */
#define NX_MOMENTARYPUSH    0
#define NX_PUSHONPUSHOFF    1
#define NX_TOGGLE           2
#define NX_SWITCH           3
#define NX_RADIOBUTTON      4
#define NX_MOMENTARYCHANGE  5

@interface ButtonCell:ActionCell
{
    int 	popup;	/* is this a popupshell buttoncell */
    char	*altContents;
    char	*icon;
    char	*altIcon;
    char        *iconFile;
    char	*altIconFile;
    int		widgetType;  /* 0=button, 1=NXRadio, 2=NXSwitch */
    int		radioBehavior;
    unsigned long	backgroundColor;
//    unsigned short	keyEquivalent;
/*
    union _icon    {
        struct _bmap    {
            id      normal;
            id      alternate;
        }
        struct _ke      {
            id      font;
            float   descent;
        }
    }  icon;
    id sound;
*/    
    struct _bcFlags1    {
        unsigned int     pushIn:1;          /* these aren't used right now...*/
        unsigned int     changeContents:1;
        unsigned int     changeBackground:1;
        unsigned int     changeGray:1;
        unsigned int     lightByContents:1;
        unsigned int     lightByBackground:1;
        unsigned int     lightByGray:1;
        unsigned int     hasAlpha:1;
        unsigned int     bordered:1;
        unsigned int     iconOverlaps:1;
        unsigned int     horizontal:1;
        unsigned int     bottomOrLeft:1;
        unsigned int     iconAndText:1;
        unsigned int     lastState:1;
        unsigned int     iconSizeDiff:1;
        unsigned int     iconIsKeyEquivalent:1;
    }          bcFlags1;
    struct _bcFlags2    {
        unsigned int     keyEquivalent:8;  /* these aren't used right now...*/
        unsigned int     transparent:1;
        unsigned int     _padding:7;
    }          bcFlags2;
}


- init;
- initTextCell:(const char *)aString;
- initIconCell:(const char *)iconName;
- copyFromZone:(NSZone *)zone;

- setIcon:(const char *)iconName;
- (const char *)icon;
- setTitle:(const char *)aString;
- (const char *)title;
- setTitleNoCopy:(const char *)aString;
- (const char *)altTitle;
- setAltTitle:(const char *)aString;
- setKeyEquivalent:(unsigned short)charCode;
- setState:(int)value;
- (const char *)icon;
- setIcon:(const char *)iconName;
- (const char *)altIcon;
- setAltIcon:(const char *)iconName;
- image;
- setImage:image;
- altImage;
- setAltImage:image;
- (int)iconPosition;
- setIconPosition:(int)aPosition;
- sound;
- setSound:aSound;
- (int)highlightsBy;
- setHighlightsBy:(int)aType;
- (int)showsStateBy;
- setShowsStateBy:(int)aType;
- setType:(int)aType;
- (BOOL)isOpaque;
- (const char *)stringValue;
- setStringValue:(const char *)aString;
- setStringValueNoCopy:(const char *)aString;
- (int)intValue;
- setIntValue:(int)anInt;
- (float)floatValue;
- setFloatValue:(float)aFloat;
- (double)doubleValue;
- setDoubleValue:(double)aDouble;
- setFont:fontObj;
- (BOOL)isBordered;
- setBordered:(BOOL)flag;
- (BOOL)isTransparent;
- setTransparent:(BOOL)flag;
- setPeriodicDelay:(float)delay andInterval:(float)interval;
- getPeriodicDelay:(float *)delay andInterval:(float *)interval;
- (unsigned short)keyEquivalent;
- setKeyEquivalent:(unsigned short)charCode;
- setKeyEquivalentFont:fontObj;
- setKeyEquivalentFont:(const char *)fontName size:(float)fontSize;
- (int)getParameter:(int)aParameter;
- setParameter:(int)aParameter to:(int)value;
- getDrawRect:(NXRect *)theRect;
- getTitleRect:(NXRect *)theRect;
- getIconRect:(NXRect *)theRect;
- calcCellSize:(NXSize *)theSize inRect:(const NXRect *)aRect;
- drawSelf:(const NXRect *)cellFrame inView:controlView;
- drawInside:(const NXRect *)aRect inView:controlView;
- highlight:(const NXRect *)cellFrame inView:controlView lit:(BOOL)flag;
- (BOOL)trackMouse:(NXEvent *)theEvent inRect:(const NXRect *)cellFrame ofView:controlView;
- performClick:sender;
- write:(NXTypedStream *)stream;
- read:(NXTypedStream *)stream;

- _setMode:(int)aMode;
- (int)_radioType;
@end

@interface ButtonCell(WidgetSet)
- _init;
- _setStringValue:(const char *)aString;
- _setState:(int)value;
- _setPopUp;
- _addCallback;
- _managedBy:parent;
@end

#endif
