/* drawing context using the Xt/Xlib library.

   Copyright (C) 1995 Free Software Foundation, Inc.

   Written by:  Adam Fedor <fedor@boulder.colorado.edu>
   Date: Nov 1995
   
   This file is part of the GNU Objective C User Interface library.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   */

/* This is a reasonable attempt at mapping PostScript operators to an
   Xt/Xlib library implementation. Don't even begin to think that this
   is a full PostScript implementation, however. Many, many operators are
   not implemented or do not work as expected. Before using an operator,
   check to make sure it is implemented, otherwise the opstack may 
   be screwed up and nothing else will work!. No errors are returned
   for non-implmeneted operators.
*/

#ifndef _XtDrawContext_h_INCLUDE
#define _XtDrawContext_h_INCLUDE

#include "DrawContext.h"
#include "xwfriends.h"

typedef enum {alloc_local, alloc_global} alloc_mode_t;

@class NSMutableArray;
@class NSMutableDictionary;
@class Stack;
@class XtGState;
@class View;

@interface XtDrawContext : DrawContext
{
    Stack *opstack;
    Stack *gstack;
    XtGState        *gstate;
    NSMutableArray  *glist;
    NSZone *localZone;
    alloc_mode_t alloc_mode;

    id fontobj;
    NSMutableDictionary *localfontdir;

    XWContext_t xwctxt;
    Region viewclip;

    /* Compatibility flags to help us out with interim widget sets */
    BOOL dontHandleEvents;
    BOOL dontHandleWidgets;
}

- initWithDisplay: (Display *)XDisplay;
- (XWContext) XWContext;
- (XtGState *) gstateObject: (int)gst;

- (void) gsaveWithView: (View *)view;
- (void) gstateWithView: (View *)view;

@end

@interface XtDrawContext (Wraps)

- (void) addwindow: (Widget)win : (float)x : (float)y : (float)w 
	: (float)h : (int)type : (int *)num;
- (void) DPSwindowdeviceview: (int)num : (void *)view;

@end

#endif /* _XtDrawContext_h_INCLUDE */
