/* XtDrawPrivate - Private definitinos for XtDrawContext files.

   Copyright (C) 1995 Free Software Foundation, Inc.

   Written by:  Adam Fedor <fedor@boulder.colorado.edu>
   Date: Nov 1995
   
   This file is part of the GNU Objective C User Interface Library.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   */

#ifndef _XtDrawPrivate_h_INCLUDE
#define _XtDrawPrivate_h_INCLUDE

#include <Foundation/NSDictionary.h>
#include <objects/Stack.h>

typedef struct _font_t 
{
  char *name;
  char *internal_name;
  XFontStruct *info;
  float size;
  char  *weight;
  char  slant;
  Font  id;
} font_t;

/* The memory zone where all global objects are allocated from (Contexts
   are also allocated from this zone) */
extern NSZone *globalZone;

/* Macros we use alot */
#define copy_string_buffer NXCopyStringBuffer

#define CTXT_OBJ_ALLOC(class) \
  [class allocWithZone: ((alloc_mode == alloc_local) ? localZone : globalZone)]

#define CHECK_NULL_OUTPUT(outvar) \
  if (outvar == NULL) \
    INVOKE_ERROR(DPSnulloutput, @"NULL output variable specified")

#define ctxt_pop(obj, stack) \
  do { \
    [self _check_stackunderflow: stack]; \
    obj = [stack popObject]; \
  } while (0)

@interface XtDrawContext (PrivateError)

- (void) _check_invalidaccess: object zone: (NSZone *)need_zone;
- (void) _check_invalidfont: (long)ident;
- (void) _check_stackunderflow: (Stack *)stack;
- (void) _check_typecheck: object class: (Class)kind;

@end

#endif
