/* Generic drawing DrawContext class.

   Copyright (C) 1995 Free Software Foundation, Inc.

   Written by:  Adam Fedor <fedor@boulder.colorado.edu>
   Date: Nov 1995
   
   This file is part of the GNU Objective C User interface library.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   */

#ifndef _dpsclient_h_INCLUDE
#define _dpsclient_h_INCLUDE

#include "dpsfriends.h"

typedef int DPSErrorCode;

typedef void (*DPSTextProc)(
  DPSContext ctxt,
  const char *buf,
  long unsigned int count );

  /* Call-back procedure to handle text from the PostScript interpreter.
     'buf' contains 'count' bytes of ASCII text. */

typedef void (*DPSErrorProc)(
  DPSContext ctxt,
  DPSErrorCode errorCode,
  long unsigned int arg1, 
  long unsigned int arg2 );
  
  /* Call-back procedure to report errors from the PostScript interpreter.
     The meaning of arg1 and arg2 depend on 'errorCode', as described above.
     See DPSDefaultErrorProc, below.
   */

extern void DPSDefaultErrorProc(
  DPSContext ctxt,
  DPSErrorCode errorCode,
  long unsigned int arg1,
  long unsigned int arg2 );
  
  /* This is the default error proc that is used for both contexts'
     individual errors and the global error backstop.  It raises an
     exception using NX_RAISE.  For all DPS errors, the context is the
     first piece of data put in the NXHandler record.  The second
     data field is value passed to this routine in arg1.
     */

extern void DPSDestroyContext(DPSContext ctxt);

  /* This calls DPSUnchainContext, then (for an interpreter context) sends a
     request to the interpreter to terminate ctxt, then frees the storage
     referenced by ctxt. The termination request is ignored by the
     server if the context is invalid, for example if it has already
     terminated. */

#include "dpsGNU.h"
  
#endif	/* GNU_DPSCLIENT_H */